from .layers import dea_c3_ls5_ard, dea_c3_ls7_ard, dea_c3_ls8_ard

folder_c3_landsat = {
        "title": "Collection 3 Landsat Surface Relectance",
        "abstract": """
        """,
        "layers": [dea_c3_ls5_ard, dea_c3_ls7_ard, dea_c3_ls8_ard],
}

