from datacube_ows.styles.base import StyleDefBase  # noqa: F401
from datacube_ows.styles.colormap import ColorMapStyleDef  # noqa: F401
from datacube_ows.styles.component import ComponentStyleDef  # noqa: F401
from datacube_ows.styles.hybrid import HybridStyleDef  # noqa: F401
from datacube_ows.styles.ramp import ColorRampDef  # noqa: F401

StyleDef = StyleDefBase
